// Copyright (C) 2022 - 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#pragma once

#include "InternalSessionFactory.h"
#include "InternalFrame.h"
#include "InternalDataBuffer.h"

namespace xma {

#define XMA_SESSION_BASE "SessionBase"

//////////////////////////////////////////////////////////////////////////////
class SessionBase : virtual public Session {
public:
  explicit SessionBase(IntSessionFactoryPtr sf, const char* logging_name, XmaParameter* params, int num_params);
  virtual ~SessionBase() {
  }

  virtual int32_t SetLog(LogPtr log) override;

  static XmaParameter* GetParameter(LogPtr log, XmaParameter* params, int num_params, const char* name);
  static void          ForEach(LogPtr log, queue_t& queue, std::function<void(IntFramePtr)> frame_func, std::function<void(IntDataBufferPtr)> buf_func);

  XmaParameter* GetParameter(XmaParameter* params, int num_params, const char* name) { return SessionBase::GetParameter(m_log, params, num_params, name); }
  int32_t       SetParam(XmaParameter* params, uint32_t param_cnt, const char* param_name, const int32_t min_value, const int32_t max_value, const int32_t default_value,
            const XmaAPIVersion earliest_version = XMA_API_VERSION_1_0, const XmaAPIVersion latest_version = XMA_API_VERSION_LATEST);
  inline void   ForEach(queue_t& queue, std::function<void(IntFramePtr)> frame_func, std::function<void(IntDataBufferPtr)> buf_func) {
    ForEach(m_log, queue, frame_func, buf_func);
  }
  void LogLatencySend(uint64_t frame);
  void LogLatencyRecv(uint64_t frame);

protected:
  const char*          m_logging_name;
  bool                 m_latency_logging;
  uint64_t             m_frame_sent = 0;
  IntSessionFactoryPtr m_sf;
  LogPtr               m_log;
};

}; // Namespace xma
