// Copyright (C) 2022 - 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#ifndef _XMA_API_H_
#define _XMA_API_H_

#include <mutex>
#include <memory>
#include "InternalSessionFactory.h"

struct XmaLogStruct {
  xma::LogPtr m_log;
};

struct XmaSessionFactory {
  xma::IntSessionFactoryPtr m_sf;
};

#define TRY() try {

#define CATCH(log, module) \
  } \
  catch (...) { \
    if (log) \
      log->LogMsg(XMA_ERROR_LOG, module, "%s() - Returning unknown error!\n", __func__); \
  }

#define CATCH_ERROR(log, module) \
  } \
  catch (std::runtime_error&) { \
    if (log) \
      log->LogMsg(XMA_ERROR_LOG, module, "%s() - Returning error!\n", __func__); \
    return XMA_ERROR; \
  } \
  catch (std::bad_alloc&) { \
    if (log) \
      log->LogMsg(XMA_ERROR_LOG, module, "%s() - Returning bad alloc!\n", __func__); \
    return XMA_ERROR_BAD_ALLOC; \
  } \
  catch (std::invalid_argument&) { \
    if (log) \
      log->LogMsg(XMA_ERROR_LOG, module, "%s() - Returning invalid argument!\n", __func__); \
    return XMA_ERROR_INVALID; \
  } \
  catch (std::out_of_range&) { \
    if (log) \
      log->LogMsg(XMA_ERROR_LOG, module, "%s() - Returning out of range!\n", __func__); \
    return XMA_ERROR_OUT_OF_RANGE; \
  } \
  catch (std::length_error&) { \
    if (log) \
      log->LogMsg(XMA_ERROR_LOG, module, "%s() - Returning length error!\n", __func__); \
    return XMA_ERROR_LENGTH_ERROR; \
  } \
  catch (...) { \
    if (log) \
      log->LogMsg(XMA_ERROR_LOG, module, "%s() - Returning unknown error!\n", __func__); \
    return XMA_ERROR; \
  }

#define CATCH_NULL(log, module) \
  } \
  catch (...) { \
    if (log) \
      log->LogMsg(XMA_ERROR_LOG, module, "%s() - Returning unknown error!\n", __func__); \
    return NULL; \
  }

#define CATCH_RET(log, module, ret) \
  } \
  catch (...) { \
    if (log) \
      log->LogMsg(XMA_ERROR_LOG, module, "%s() - Returning unknown error!\n", __func__); \
    return ret; \
  }

#endif
